/**
 * @file
 * 
 * Sets the events to enable links to be opened in modalframes.
 * 
 * In this module, links to be opened in modalframes are designated 
 * by addition of the class "namecards-open-in-modalframe-XxY" 
 * where X and Y are integers representing the width (X) and height 
 * (Y) of the modalframe in pixels.      
 */

(function ($) {
  Drupal.behaviors.namecardsModalFrame = function() {
	// Add 'namecards-open-in-modalframe-' class to the 'Add' new namecard link. 
	// This class enables the link to be displayed in a modalframe, as well as 
	// defining the dimensions of the modalframe. Changes to the modelframe 
	// dimensions should be made here by modifying the following two variables.
	var addNewContactModalframeX = 600;
	var addNewContactModalframeY = 800;
	$('ul.tabs li a[href="/namecards/add_contact"]:not(.namecards-modalframe-processed)').addClass('namecards-open-in-modalframe-' + addNewContactModalframeX + 'x' + addNewContactModalframeY);
	
	// Respond to click event on links which should open in modalframes
	$('a[class*="namecards-open-in-modalframe-"]:not(.namecards-modalframe-processed)').addClass('namecards-modalframe-processed').click(function() {
      var element = $(this);
      
      // Gets the x,y dimensions of the modalframe based on the dimensions specified in the class name.
      // @return dimension
      //   Associative array containing the width (x) and height (y) of the modalframe in pixels.
      function getModalFrameDimensions() {
    	var elementClass = element.attr('class'); // Var element is inherited from bounding function.
    	var dimension = new Array(); // Associative array containing values of x,y
    	var objRegExp;
    	var temp;
    	
    	objRegExp = 'namecards-open-in-modalframe-[0-9]{1,}x[0-9]{1,}';
    	temp = elementClass.match(objRegExp);
    	temp = String(temp).match('[0-9]{1,}x[0-9]{1,}$');
        temp = String(temp).split('x');
    	
        dimension['x'] = parseInt(temp[0]);
        dimension['y'] = parseInt(temp[1]);
        
    	return dimension;
      }
      
      // This is our onSubmit callback that will be called from the child window
      // when it is requested by a call to modalframe_close_dialog() performed
      // from server-side submit handlers.
      function onSubmitCallbackNamecards(args, statusMessages) {
        // Display status messages generated during submit processing.
        if (statusMessages) {
          //$('.namecards-modalframe-messages').hide().html(statusMessages).show('slow');
        }
        // Reload page if reload flag has been set 
        if (args && args.reload_page) {
          location.href = args.reload_page;
       }
      }

      // Hide the messages are before opening a new dialog.
      $('.namecards-modalframe-messages').hide('fast');

      // Get dimensions of the modalframe.
      frameDimension = getModalFrameDimensions();
      
      // Build modal frame options.
      var modalOptions = {
        url: $(element).attr('href'),
        autoFit: true,
        width: frameDimension['x'],
        height: frameDimension['y'],
        onSubmit: onSubmitCallbackNamecards
      };

      // Open the modal frame dialog.
      Drupal.modalFrame.open(modalOptions);

      // Prevent default action of the link click event.
      return false;
    });
  };
})(jQuery);
