<?php

/**
 * @file
 * This file contains functions pertaining to creating email lists for module namecards
 *
 * Copyright 2010 Ben Gunn
 *
 * Licensed under the GNU Public License
 */

/**
 * Loads namecards_email_list_form
 */
function namecards_email_list() {
  $output = '';

  // Embed modalframe messages div.
  $output .= '<div class="namecards-modalframe-messages"></div>';

  // Embed email list form.
  $output .= drupal_get_form('namecards_email_list_form');

  return $output;
}

/**
 * Create page for menu item 'namecards/email_list'.
 */
function namecards_email_list_form(&$form_state) {
  $show_create_email_list_form = (isset($form_state['storage']['show_create_email_list_form']) ? $form_state['storage']['show_create_email_list_form'] : TRUE);

  if ($show_create_email_list_form) {

    // Add search box.
    $form['namecards_live_search_textfield'] = array(
      '#type' => 'textfield',
      '#default_value' => (!empty($form_state['storage']['email_list_form_search_terms'])) ? $form_state['storage']['email_list_form_search_terms'] : '',
      '#size' => 30,
      '#maxlength' => 400,
    );
    $form['namecards_live_search_search_button'] = array(
      '#type' => 'submit',
      '#value' => t('Search'),
    );
    $form['namecards_live_search_clear_button'] = array(
      '#type' => 'submit',
      '#value' => t('Clear'),
    );

    // Create where clause string in order to filter by search terms.
    $search_term_where_clause = ' ';

    if (!empty($form_state['storage']['email_list_form_search_terms'])) {
      $search_terms = array();
      // Extract search terms.
      $search_terms = explode(' ', check_plain(trim($form_state['storage']['email_list_form_search_terms'])));

      $search_term_where_clause .= 'AND ';
      for ($i = 0; $i < count($search_terms); $i++) {
        // Build the where clause.
        // TODO: There is a strange bug in which the search terms are not
        // converted to upper in the sql statement. Only seems to apply
        // to nodes which have been imported.  Nodes created by the add
        // contact form appear uneffected. As a temporary solution I convert
        // the search terms to upper case using php rather than relying on
        // database to do it.
        if ($i < count($search_terms) - 1) {
          $search_term_where_clause .= "UPPER(ctnn.field_computed_full_content_value) LIKE UPPER('%" . db_escape_string(strtoupper($search_terms[$i])) . "%') AND ";
        }
        else {
          $search_term_where_clause .= "UPPER(ctnn.field_computed_full_content_value) LIKE UPPER('%" . db_escape_string(strtoupper($search_terms[$i])) . "%')";
        }
      }
    }

    $sql = "SELECT
              n.nid AS field_nid,
              n.title AS field_surname,
              ctnn.field_given_name_value AS field_given_name,
              ctnn.field_nickname_value AS  field_nickname,
              ctnn.field_computed_org_name_value AS field_org_name,
              ctnn.field_computed_email_value AS field_computed_email
            FROM
              {node} n
            LEFT JOIN {content_type_namecard_namecard} ctnn ON n.nid = ctnn.nid
            WHERE n.type = 'namecard_namecard'
            AND ctnn.field_computed_email_value <> ''
            " . $search_term_where_clause . "
            ORDER BY field_surname ASC, field_given_name ASC, field_org_name ASC";
    $results = db_query($sql);
    $checkboxes = array();

    while ($result = db_fetch_object($results)) {
      $checkboxes[$result->field_nid] = '';
      $form[$result->field_nid]['surname'] = array(
        '#value' => check_plain($result->field_surname),
      );
      $form[$result->field_nid]['given_name'] = array(
        '#value' => check_plain($result->field_given_name),
      );
      $form[$result->field_nid]['nickname'] = array(
        '#value' => check_plain($result->field_nickname),
      );
      $form[$result->field_nid]['org_name'] = array(
        '#value' => check_plain($result->field_org_name),
      );
      $form[$result->field_nid]['email'] = array(
        '#value' => check_plain($result->field_computed_email),
      );
    }

    $form['checkboxes'] = array(
      '#type' => 'checkboxes',
      '#options' => $checkboxes,
      '#default_value' => (isset($form_state['storage']['nids'])) ? $form_state['storage']['nids'] : array(),
    );

    // Only display createlist widgets if there are contacts with email addresses.
    if (count($checkboxes) > 0) {
      $form['create_list'] = array(
        '#type' => 'fieldset',
        '#title' => t('Create mailing list'),
        '#collapsible' => FALSE,
        '#access' => TRUE,
      );
      $form['create_list']['email_client_type'] = array(
        '#type' => 'select',
        '#title' => t('Email program'),
        '#options' => array(
          1 => 'MS Outlook',
          2 => 'Thunder Bird',
        ),
        '#default_option' => 1,
        '#description' => t('Select the email program you will use to send this mailing list'),
      );
      $form['create_list']['submit'] = array(
        '#type' => 'submit',
        '#value' => t('Create'),
      );
    }

    $form['#theme'] = 'namecards_create_email_list_theme';

    // Set session var for displaying search textbox.  This would be better placed
    // in form submit function but looks like session vars cannot be set from the
    // form submit function. So we place here instead.
    $_SESSION['namecards_email_list_show_search_box'] = 0;
  }
  else {
    // Get nids of selected contacts.
    $selected_nids = array_filter($form_state['values']['checkboxes'], '_namecards_create_email_list_get_nids');

    if (!empty($selected_nids)) {
      $sql = "SELECT
                cfe.field_email_value AS email
              FROM
                {content_field_email} cfe
              WHERE
                cfe.nid IN (" . db_placeholders($selected_nids, 'int') . ")";

      $results = db_query($sql, $selected_nids);
    }

    // Set email client name string and format of email address separation character
    switch ((int)$form_state['values']['email_client_type']) {
      case 1:
        // MS Outlook
        $email_client_name = "Microsoft Outlook"; //Set text to appear in email list title
        $sep_char = ';'; //Set separation char for email list
        break;
      case 2:
        // Thunder bird
        $email_client_name = "Thunderbird"; //Set text to appear in email list title
        $sep_char = ',';  //Set separation char for email list
        break;
    }

    // Process email addresses to create final email address list content formatted for specified email client type.
    $email_addresses = array();
    while ($result = db_fetch_object($results)) {
      $email_addresses[] = $result->email;
    }
    $email_list_content = '';
    $number_of_email_addresses = count($email_addresses);
    for ($i = 0; $i < $number_of_email_addresses; $i++) {
      if ($i!= $number_of_email_addresses - 1) { // Is not the final email address so add seperator
        $email_list_content .= $email_addresses[$i] . $sep_char .' ';
      }
      else { // Is final email address so don't add separator
        $email_list_content .= $email_addresses[$i];
      }
    }

    // Add form elements.
    $form['email_list'] = array(
      '#title' => t('Email mailing list'),
      '#type' => 'textarea',
      '#description' => t('Mailing list formatted for use with @email_client.  To use simply copy then paste this list into the recipient section of your email message.', array('@email_client' => $email_client_name)),
      '#default_value' => $email_list_content,
      '#resizable' => TRUE,
      '#rows' => 5,
      '#attributes' => array(
        'id' => 'namecards-email-list-content-textarea',
      ),
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Back'),
    );

    // Store the nids of the selected nodes.  These will be used to rebuild the create email list form.
    $form_state['storage']['nids'] = $selected_nids;

    // Set session var for displaying search textbox.  This would be better placed
    // in form submit function but looks like session vars cannot be set from the
    // form submit function. So we place here instead.
    $_SESSION['namecards_email_list_show_search_box'] = 1;
  }

  return $form;
}

/**
 * Form submit function
 */
function namecards_email_list_form_submit($form, &$form_state) {
  switch ($form_state['clicked_button']['#value']) {
    case t('Back'):
      $form_state['storage']['show_create_email_list_form'] = TRUE;
      break;
    case t('Search'):
      $form_state['storage']['email_list_form_search_terms'] = $form_state['values']['namecards_live_search_textfield'];
      break;
    case t('Clear'):
      $form_state['storage']['email_list_form_search_terms'] = NULL;
      break;
    case t('Create'):
      $form_state['storage']['show_create_email_list_form'] = FALSE;
      break;
    default:
      break;
  }
}

/**
 * Email list form validation
 */
function namecards_email_list_form_validate($form, &$form_state) {
  if ($form_state['clicked_button']['#value'] == t('Create')) {
    $selected_nids = array_values(array_filter($form_state['values']['checkboxes']));
    if (count($selected_nids) < 1) {
      form_set_error('checkboxes', t('You must select at least one contact from the provided list.'));
    }
  }
}

/**
 * Callback function used in call to filter_array().
 *
 * Check if form checkbox is selected by evaluating whether checkbox
 * value is an integer greater than zero (i.e. is selected).
 * @see namecards_email_list_form().
 */
function _namecards_create_email_list_get_nids($var) {
  if (is_int($var) && $var > 0) {
    return TRUE;
  }
  else {
    return FALSE;
  }
}

/**
 * Theme create email list form
 */
function theme_namecards_create_email_list_theme($form) {
  $rows = array();
  foreach (element_children($form['checkboxes']) as $uid) {
    $row = array();
    $row[] = drupal_render($form['checkboxes'][$uid]);
    $row[] = drupal_render($form[$uid]['surname']);
    $row[] = drupal_render($form[$uid]['given_name']);
    $row[] = drupal_render($form[$uid]['nickname']);
    $row[] = drupal_render($form[$uid]['org_name']);
    $row[] = drupal_render($form[$uid]['email']);
    $rows[] = $row;
  }

  if (count($rows)) {
    $header = array(theme('namecards_select_table_column_checkbox'), t('Surname'), t('Given name'), t('Nickname'), t('Organization'), t('Email'));
  }
  else {
    $header = array(t('Surname'), t('Given name'), t('Nickname'), t('Email'));
    $row = array();
    $row[] = array(
      'data' => '<strong>' . t('No contacts with email addresses were found') . '</strong>',
      'colspan' => 4,
      'style' => 'text-align:center'
    );
    $rows[] = $row;
  }

  // Set table tag attributes
  $attributes = array(
    'class' => 'namecards-create-email-list-table',
  );

  $output = '<div class="namecards-create-email-list-wrapper">';
  $output .= drupal_render($form['namecards_live_search_textfield']);
  $output .= drupal_render($form['namecards_live_search_search_button']);
  $output .= drupal_render($form['namecards_live_search_clear_button']);
  $output .= theme('table', $header, $rows, $attributes);
  $output .= drupal_render($form);
  $output .= '</div>';
  return $output;
}